%{
===========================================================================
Code provided by Yichuan (Charlie) Tang
Permission is granted for anyone to copy, use, modify, or distribute this
program and accompanying programs and documents for any purpose, provided
this copyright notice is retained and prominently displayed, along with
a note saying that the original programs are available from our
web page.
The programs and documents are distributed without any warranty, express or
implied.  As the programs were written for research purposes only, they 
have not been tested to the degree that would be advisable in any important
application.  All use of these programs is entirely at the user's own risk.
===========================================================================

CT 3/2012 - Sampling from a Mixture of Tensor Analyzer
conditioned on z_c

MoTA is a struct with the fields:
     M (D x C)     - k d-dimensional means
     W (D x (d1+d2+d3) x C) - 2-biases
     Psi (D,C)     - diagonal noise levels
     T - tensor (matricized) D x (d1*d2) x C
     mix - C x 1 mixing proportions
     .d - dimensionality = [d1=2 d2=5 d3=10]

zs = cell(C, d), each cell is dim by N.
c - which component;
bNoise - 1 means we want to add noise, 0 means otherwise   
xx - N by dim by N sample matrix
%}

function [xx] = mta3_cond_zs( MoTA, zs, c, bNoise)

assert( length(MoTA.d) == size(zs,2) );
assert( length(MoTA.d) >= 2 );

D = size(MoTA.M,1);
N = size(zs{c,1},2);
xx = zeros(D,N);

setinds = [];
setinds{1} = 2:length(MoTA.d);
setinds{2} = 1;  %let's assume the first latent factor is "this z"

% inds into the W required.
this_z_inds = false(1, sum(MoTA.d));
rest_z_inds = false(1, sum(MoTA.d));

ind = 1;
for b = 1:length(MoTA.d)
    if b == setinds{2}
        this_z_inds(ind:ind+MoTA.d(b)-1) = true;      
        rest_z_inds(ind:ind+MoTA.d(b)-1) = false;
    else
        this_z_inds(ind:ind+MoTA.d(b)-1) = false;
        rest_z_inds(ind:ind+MoTA.d(b)-1) = true;
    end
    ind = ind + MoTA.d(b);
end
 
Wconcate  = MoTA.W(:,:,c); %concatenated W
%split W:
W_this_z = Wconcate(:, this_z_inds);    %2-biases
W_rest_z = Wconcate(:, rest_z_inds);    %to be multiplied with z_given
Psi   = MoTA.Psi(:,c);

y = cat(1, zs{c,setinds{1}});

z_c = cell(length(MoTA.d),1);
[z_c{:}] = zs{c,:};
W_batch = bsxfun(@plus, W_this_z, TensorMatrixProd_zs( MoTA.T(:,:,c), MoTA.d, z_c, setinds{1}, setinds{2} ));
M = bsxfun(@plus, MoTA.M(:,c), W_rest_z*y);
%W, m are the MFA parameters conditioned on z2
assert( size(W_batch,3) == N && size(M,2) == N);
assert( size(zs{c,setinds{2}},2) == N);
for n = 1:size(zs{c,setinds{2}},2)
    xx(:,n) = bsxfun(@plus, M(:,n), W_batch(:,:,n)*zs{setinds{2},c}(:,n))+  bNoise*spdiags( Psi.^(1/2),0,D,D)*randn(D, 1);
end












